var upContent = {'name': '..', 'ext': 'dir', 'size': ''}
var globalContent = null;
var currentPath = '';
var fileListTableId = 'fileListTable';
var renameButtonId = 'renameButton';
var loadFileButtonId = 'loadFileButton';
var deleteButtonId = 'deleteButton';
var selectAllButtonId = 'selectAllButton';
var unselectAllButtonId = 'unselectAllButton';
var uploadFilesButtonId = 'uploadFilesButton';
var createFolderButtonId = 'createFolderButton';

function fileSizeStr(size)
{
    var idx = 0;
    while (size > 1024 &&
           idx < 4) {
        size /= 1024;
        ++idx;
    }
    var str;
    switch (idx) {
        case 0:
            str = 'b';
            break;
        case 1:
            str = 'Kb';
            break;
        case 2:
            str = 'Mb';
            break;
        default:
            str = 'Gb';
    }
    var roundedSize = Math.floor(size);
    if (size - roundedSize < 0.1)
        return roundedSize + ' ' + str;
    else
        return size.toFixed(1) + ' ' + str;
}

function normalizeFileSize()
{
    for (var i = 0; i < globalContent.length; ++i)
        if (globalContent[i].ext != 'dir')
            globalContent[i].size = fileSizeStr(globalContent[i].size);
}

function checkFilename(filename)
{
    if (filename.length == 0)
        return false;
    for (var i = 0; i < filename.length; ++i)
    {
        if (filename[i] == '/' || filename[i] == '\\')
            return false;
    }
    return true;
}

//requests

function createXMLHttpRequest(completeEvent)
{
	var xmlhttp = new XMLHttpRequest();
	if (completeEvent)
	{
		xmlhttp.addEventListener("load", 
		function()
		{
			if (xmlhttp.status != 200)
				completeEvent(false, requestErrorDialogTitle);
			else
			{
				if (xmlhttp.responseText == '' || xmlhttp.responseText == null)
					completeEvent(true);
				else
				{
					var responseObject = JSON.parse(xmlhttp.responseText);
					completeEvent(responseObject.result, responseObject.message, responseObject.data);
				}
			}
		}, false);
		
		xmlhttp.addEventListener("error", 
		function()
		{
			completeEvent(false, requestErrorDialogTitle);
		}, false);
		
		xmlhttp.addEventListener("abort", 
		function()
		{
		}, false);
	}
	return xmlhttp;
}

function sendRequest(request, completeEvent)
{
	var xmlhttp = createXMLHttpRequest(completeEvent);
	xmlhttp.open('GET', request, true);
	xmlhttp.send(null);
}

function getContentRequest(path, completeEvent)
{
	var request = '/getcontent?path=' + encodeURIComponent(path);
	sendRequest(request, completeEvent);
}

function renameRequest(path, newPath, completeEvent)
{
	var request = '/rename?path=' + encodeURIComponent(path) + '&newpath=' + encodeURIComponent(newPath);
	sendRequest(request, completeEvent);
}

function deleteRequest(path, completeEvent)
{
	var request = '/delete?path=' + encodeURIComponent(path);
	sendRequest(request, completeEvent);
}

function createDirRequest(path, completeEvent)
{
	var request = '/createdir?path=' + encodeURIComponent(path);
	sendRequest(request, completeEvent);
}

function fileExistsRequest(path, completeEvent)
{
	var request = '/fileexists?path=' + encodeURIComponent(path);
	sendRequest(request, completeEvent);
}

function getFileRequest(path)
{
	window.open('/getfile?path=' + encodeURIComponent(path));
}

function createFileRequest(path, file, completeEvent, progressEvent) 
{
	var request = "/createfile?path=" + encodeURIComponent(path) + "&size=" + file.size;
	var xmlhttp = createXMLHttpRequest(completeEvent);
	if (progressEvent != null)
		xmlhttp.upload.addEventListener('progress', progressEvent, false);
	var formData = new FormData();
	xmlhttp.open("POST", request, true);
	formData.append('MyFile', file);
	xmlhttp.send(formData);
	
	var controlLambda = 
	function()
	{
		xmlhttp.abort();
	};
	
	return controlLambda;
}


function sortContent(a, b)
{
	if (a.name == '..')
		return -1;
	else if (b.name == '..')
		return 1;
	else if (a.ext == 'dir' && b.ext != 'dir')
		return -1;
	else if (b.ext == 'dir' && a.ext != 'dir')
		return 1;
	else if ((a.name + a.ext) < (b.name + b.ext))
		return -1;
	else return 1;
}

function drawContent()
{
	var fieldList = [{'name': 'name', 'value': tableNameFieldTitle}, 
					{'name': 'ext', 'value': tableExtFieldTitle}, 
					{'name': 'size', 'value': tableSizeFieldTitle}];
	var eventList = [{'eventType': 'click', 'eventHandler': onRowClick}, 
					{'eventType': 'dblclick', 'eventHandler': onRowDblClick}];
	fillTable(fileListTableId, fieldList, globalContent, 0, null, eventList, null);
	document.getElementById(renameButtonId).disabled = true;
	document.getElementById(loadFileButtonId).disabled = true;
    document.getElementById(deleteButtonId).disabled = true;
    document.getElementById(unselectAllButtonId).disabled = true;
    document.getElementById(uploadFilesButtonId).disabled = false;
    document.getElementById(createFolderButtonId).disabled = false;
}

function loadContent(path)
{
	getContentRequest(path,
	function(requestResult, requestMessage, requestData)
	{
		if (requestResult)
		{
			currentPath = path;
			globalContent = requestData;
			if (currentPath != '')
				globalContent.push(upContent);
			globalContent.sort(sortContent);
            normalizeFileSize();
			drawContent();
		}
		else messageDialog(requestMessage); 
	});
}

function updateButtonsState()
{
    var selectedRowIdList = getSelectedRowIdList(fileListTableId);
    document.getElementById(renameButtonId).disabled = (selectedRowIdList.length != 1);
    document.getElementById(deleteButtonId).disabled = (selectedRowIdList.length == 0);
    document.getElementById(selectAllButtonId).disabled = (selectedRowIdList.length == globalContent.length);
    document.getElementById(unselectAllButtonId).disabled = (selectedRowIdList.length == 0);
    document.getElementById(uploadFilesButtonId).disabled = (selectedRowIdList.length != 0);
    document.getElementById(createFolderButtonId).disabled = (selectedRowIdList.length != 0);
    
    var loadFileButtonDisabled = selectedRowIdList.length == 0;
    for (var i = 0; i < selectedRowIdList.length; ++i)
        if (globalContent[getRowIndex(selectedRowIdList[i])].ext == 'dir')
        {
            loadFileButtonDisabled = true;
            break;
        }
    document.getElementById(loadFileButtonId).disabled = loadFileButtonDisabled;
}

function setSelectedAll(selectedValue)
{
	var rowIdList = getRowIdList(fileListTableId);
	if (rowIdList == null)
		return;
	for (var i = 0; i < rowIdList.length; ++i)
		if (globalContent[getRowIndex(rowIdList[i])].name != upContent.name)
			setRowSelected(rowIdList[i], selectedValue);
    updateButtonsState();
}

//table events
function onRowClick(params, event)
{
	if (globalContent[params.rowIndex].name != upContent.name)
		setRowSelected(params.rowId, !getRowSelected(params.rowId));
    updateButtonsState();
}

function onRowDblClick(params, event)
{
	var subContent = globalContent[params.rowIndex];
	if (subContent.ext == 'dir')
	{
		var path = (subContent.name == upContent.name) ? 
			currentPath.substring(0, currentPath.lastIndexOf('/')) : 
			currentPath + '/' + subContent.name;
		loadContent(path);
	}
	else
	{
		var path = currentPath + '/' + subContent.name + '.' + subContent.ext;
		getFileRequest(path);
	}
}

//buttons click events
function renameButtonClick()
{ 
	var selectedRowIdList = getSelectedRowIdList(fileListTableId);
	var subContent = globalContent[getRowIndex(selectedRowIdList[0])];
	inputStringDialog(enterNameDialogTitle, subContent.name, 
	function(dialogResult, dialogValue)
	{
		if (!dialogResult)
			return;
        if (!checkFilename(dialogValue))
        {
            messageDialog("Incorrect name", function(){});
            return;
        }
		var ext = ((subContent.ext == 'dir') ? '' : ('.' + subContent.ext));
		var path = currentPath + '/' + subContent.name + ext;
		var newPath = currentPath + '/' + dialogValue + ext;
		renameRequest(path, newPath, 
		function(requestResult, requestMessage)
		{
			if (requestResult)
			{
				subContent.name = dialogValue;
				globalContent.sort(sortContent);
				drawContent();
			}
			else messageDialog(requestMessage); 
		});
	});	
}
function deleteSubContent(subContent)
{
	var ext = ((subContent.ext == 'dir') ? '' : ('.' + subContent.ext));
	var path = currentPath + '/' + subContent.name + ext;
	deleteRequest(path, 
	function(requestResult, requestMessage)
	{
		if (requestResult)
		{
			for (var j = 0; j <  globalContent.length; ++j)
				if ( globalContent[j].name == subContent.name &&  globalContent[j].ext == subContent.ext)
				{
					globalContent.splice(j, 1);
					break;
				}
			drawContent();
		}
		else messageDialog(requestMessage);
	});
}

function deleteButtonClick()
{
	var selectedRowIdList = getSelectedRowIdList(fileListTableId);
	if (selectedRowIdList.length == 0)
		return;
	confirmDialog(deleteContentDialogTitle, 
	function(dialogResult)
	{
		if (!dialogResult)
			return;
		var subContentList = [];
		for (var i = 0; i < selectedRowIdList.length; ++i)
			subContentList.push(globalContent[getRowIndex(selectedRowIdList[i])]);
		for (var i = 0; i < subContentList.length; ++i)
			deleteSubContent(subContentList[i])
	});
}

function createFolderButtonClick()
{
	inputStringDialog(enterFolderNameDialogTitle, 'folder', 
	function(dialogResult, dialogValue)
	{
		if (!dialogResult)
			return;
        if (!checkFilename(dialogValue))
        {
            messageDialog("Incorrect folder name", function(){});
            return;
        }
		var path = currentPath + '/' + dialogValue;
		createDirRequest(path, 
		function(requestResult, requestMessage)
		{
			if (requestResult)
			{
				var folderContent = {'name': dialogValue, 'ext': 'dir', 'size': ''};
				globalContent.push(folderContent);
				globalContent.sort(sortContent);
				drawContent();
			}
			else messageDialog(requestMessage); 
		});
	});
}

function uploadFileList(fileList, fileIndex)
{
    if (fileIndex >= fileList.length)
        return;
	
	var file = fileList[fileIndex];
	var path = currentPath + '/' + file.name;	
	
	if (file.size > 4*1024*1024*1024)
	{
		messageDialog(file.name + ': ' + largeFileErrorDialogTitle, 
		function()
		{
			uploadFileList(fileList, ++fileIndex);
		});
		return;
	}

	fileExistsRequest(path,
	function(requestResult, requestMessage)
	{
		if (!requestResult)
		{
			var createFileAbortControl = null;
			
			var progressControl = progressDialog(uploadFileDialogTitle + ': ' + file.name, 
			function()
			{
				createFileAbortControl();
			});
	 
			createFileAbortControl = createFileRequest(path, file, 
			function(requestResult, requestMessage)
			{
				progressControl(true);
		
				if (requestResult)
				{
					var shortName = '';
					var ext = '';
					var splitterIndex = file.name.lastIndexOf('.');
					if (splitterIndex != -1)
					{
						shortName = file.name.substring(0, splitterIndex);
						ext = file.name.substring(++splitterIndex);
					}
					else shortName = file.name;
					var fileContent = {'name': shortName, 'ext': ext, 'size': fileSizeStr(file.size)};
							  
					globalContent.push(fileContent);
					globalContent.sort(sortContent);
					drawContent();
							  
					uploadFileList(fileList, ++fileIndex);
				}
				else 
				{
					messageDialog(requestMessage, function(){});
				}
			},
			function(progressInfo)
			{
				if (progressInfo.lengthComputable)
					progressControl(false, progressInfo.loaded, progressInfo.total);
			});
		}
		else
		{
			messageDialog(requestMessage,
			function()
			{
				uploadFileList(fileList, ++fileIndex);
			});
		}
	});
}

function uploadFilesButtonClick()
{
	var hideInput = $('<input>');
	hideInput.attr({'type': 'hidden', 'class': 'hiddenInputFileOverlay'});
	$('body').append(hideInput);
	var openFileInput = $('<input>');
	openFileInput.attr({'type': 'file', 'multiple': 'multiple'});
	openFileInput.on('change', 
	function()
	{
		uploadFileList(openFileInput.get(0).files, 0);
		openFileInput.remove();
		hideInput.remove();
	});
	hideInput.append(openFileInput);
	openFileInput.click();
}
function loadFileButtonClick()
{
	var selectedRowIdList = getSelectedRowIdList(fileListTableId);
	for (var i = 0; i < selectedRowIdList.length; ++i)
	{
		var subContent = globalContent[getRowIndex(selectedRowIdList[i])];
		var path = currentPath + '/' + subContent.name + '.' + subContent.ext;
		getFileRequest(path);
	}
}
function selectAllButtonClick()
{ 
	setSelectedAll(true);
}

function unselectAllButtonClick()
{ 
	setSelectedAll(false);
}

//start script point
$(document).ready(function() 
{
	document.getElementById('loadFileButton').value = loadFileButtonTitle;
	document.getElementById('uploadFilesButton').value = uploadFilesButtonTitle;
	document.getElementById('createFolderButton').value = createFolderButtonTitle;
	document.getElementById('renameButton').value = renameButtonTitle;
	document.getElementById('deleteButton').value = deleteButtonTitle;
	document.getElementById('selectAllButton').value = selectAllButtonTitle;
	document.getElementById('unselectAllButton').value = unselectAllButtonTitle;
	loadContent('');
});