//global variables
var cellClassPrefix = 'cell';
var rowClassPrefix = 'row';
var tableClassPrefix = 'table';
var headerClassPrefix = 'header';
var fieldClassPrefix = 'field';
var cellIdIndex = 2;
var rowIdIndex = 1;
var tableIdIndex = 0;

//table private methods
function attachEvent(element, eventType, eventHandler, eventParams)
{
	element.on(eventType, function(event){eventHandler(eventParams, event);});
}

function attachEventList(element, eventList, eventParams)
{
	if (eventList != null)
		for (var i = 0; i < eventList.length; ++i)
			attachEvent(element, eventList[i].eventType, eventList[i].eventHandler, eventParams);
}

function createCell(field, cellContent, tableId, rowId, rowIndex, cellIndex, cellEventList)
{
	var cellId = rowId + '_' + cellIndex;
	var eventParams = {'tableId': tableId, 'rowId': rowId, 'rowIndex': rowIndex, 'cellId': cellId, 'cellIndex': cellIndex};
	var cell = $('<td>');
	cell.attr({'class': cellClassPrefix, 'id': cellId, 'field': field.name, 'index': cellIndex});
	attachEventList(cell, cellEventList, eventParams);
	cell.html(cellContent);
	return cell;
}

function createField(fieldContent, rowId, fieldIndex)
{
	var fieldId = rowId + '_' + fieldIndex;
	var field = $('<td>');
	field.attr({'class': fieldClassPrefix, 'id': fieldId});
	field.html(fieldContent);
	return field;
}

function createHeder(fieldList, tableId)
{
	var headerId = tableId + '_-1';
	var header = $('<tr>');
	header.attr({'class': headerClassPrefix, 'id': headerId});
	for (var i = 0; i < fieldList.length; ++i)
		header.append(createField(fieldList[i].value, headerId, i));
	return header;
}

function createRow(fieldList, rowContent, tableId, rowIndex, rowEventList, cellEventList)
{
	var rowId = tableId + '_' + rowIndex;
	var eventParams = {'tableId': tableId, 'rowId': rowId, 'rowIndex': rowIndex};
	var row = $('<tr>');
	row.attr({'class': rowClassPrefix, 'id': rowId, 'selected': false, 'index': rowIndex});
	attachEventList(row, rowEventList, eventParams);
	for (var i = 0; i < fieldList.length; ++i)
		row.append(createCell(fieldList[i], rowContent[fieldList[i].name], tableId, rowId, rowIndex, i, cellEventList));
	return row;
}

//table public methods
function fillTable(tableId, fieldList, tableContent, tableEventList, rowEventList, cellEventList)
{
	var eventParams = {'tableId': tableId};
	var table = $('#' + tableId);
	table.empty();
	table.attr({'class': tableClassPrefix});
	attachEventList(table, tableEventList, eventParams);
	table.append(createHeder(fieldList, tableId));
	for (var i = 0; i < tableContent.length; ++i)
		table.append(createRow(fieldList, tableContent[i], tableId, i, rowEventList, cellEventList));
}
function getRowSelected(rowId){return $('#' + rowId).attr('selected');}
function setRowSelected(rowId, selectValue){$('#' + rowId).attr('selected', selectValue);}
function getRowIdList(tableId)
{
	var rowList = $("[class=" + rowClassPrefix + "][id^='" + tableId + "_']");
	if (rowList == null)
		return null;
	var rowIdList = [];
	for (var i = 0; i < rowList.length; ++i)
		rowIdList.push(rowList[i].id);
	return rowIdList;
}
function getSelectedRowIdList(tableId)
{
	var rowList = $("[class=" + rowClassPrefix + "][selected][id^='" + tableId + "_']");
	if (rowList == null)
		return null;
	var rowIdList = [];
	for (var i = 0; i < rowList.length; ++i)
		rowIdList.push(rowList[i].id);
	return rowIdList;
}
function getRowIndex(rowId)
{
	return $('#' + rowId).attr('index');
}